package com.smartgwt.sample.client;

import com.google.gwt.core.client.EntryPoint;
import com.smartgwt.client.core.KeyIdentifier;
import com.smartgwt.client.data.DataSource;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.rpc.Messaging;
import com.smartgwt.client.rpc.MessagingCallback;
import com.smartgwt.client.rpc.RPCCallback;
import com.smartgwt.client.rpc.RPCRequest;
import com.smartgwt.client.rpc.RPCResponse;
import com.smartgwt.client.types.Overflow;
import com.smartgwt.client.types.SelectionStyle;
import com.smartgwt.client.types.SortArrow;
import com.smartgwt.client.util.KeyCallback;
import com.smartgwt.client.util.Page;
import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.Canvas;
import com.smartgwt.client.widgets.IButton;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.form.DynamicForm;
import com.smartgwt.client.widgets.form.fields.ButtonItem;
import com.smartgwt.client.widgets.form.fields.TextAreaItem;
import com.smartgwt.client.widgets.form.fields.TextItem;
import com.smartgwt.client.widgets.form.fields.events.ClickEvent;
import com.smartgwt.client.widgets.form.fields.events.ClickHandler;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.widgets.grid.ListGridRecord;
import com.smartgwt.client.widgets.grid.events.RecordClickEvent;
import com.smartgwt.client.widgets.grid.events.RecordClickHandler;
import com.smartgwt.client.widgets.layout.HLayout;
import com.smartgwt.client.widgets.layout.VStack;
import com.smartgwt.client.widgets.viewer.DetailViewer;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 */
public class SimpleChat implements EntryPoint {
	private Canvas chatLog;
    private DynamicForm chatForm;

    /**
     * This is the entry point method.
     */
    public void onModuleLoad() {
        KeyIdentifier debugKey = new KeyIdentifier();
        debugKey.setCtrlKey(true);
        debugKey.setKeyName("D");

        Page.registerKey(debugKey, new KeyCallback() {
            public void execute(String keyName) {
                SC.showConsole();
            }
        });
        
        chatLog = new Canvas();
        chatLog.setBackgroundColor("white");
        chatLog.setBorder("2px solid gray");
        chatLog.setContents("Chat Session<br>Open this page in multiple client browsers for multi-user chat.<br><br>");
        chatLog.setHeight(200);
        chatLog.setLeft(50);
        chatLog.setOverflow(Overflow.AUTO);
        chatLog.setTop(50);
        chatLog.setWidth(500);
        
        chatLog.draw();
        
        chatForm = new DynamicForm();
        chatForm.setLeft(50);
        chatForm.setTop(275);
        
        TextItem userName = new TextItem("user");
        userName.setRequired(true);
        userName.setTitle("User Name");
        
        TextAreaItem messageArea = new TextAreaItem("msg");
        messageArea.setHeight(50);
        messageArea.setTitle("Message");
        messageArea.setWidth(400);
        
        ButtonItem send = new ButtonItem();
        send.setTitle("Send");
        send.setColSpan("*");
        send.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				sendChatMessage();
				
			}
        	
        });
        
        chatForm.setItems(userName, messageArea, send);
        
        chatForm.draw();
        
        // And actually subscribe to messaging:
        Messaging.subscribe("chatChannel", new MessagingCallback() {

			@Override
			public void execute(Object data) {

				chatLog.setContents(chatLog.getContents() + (String)data);
			}
			
		});
    }
    
    public void sendChatMessage() {


            if (!chatForm.validate()) return;
            
            String userName = (String) chatForm.getValue("user");
            Object messageText = chatForm.getValue("msg");
            
            
            // If no message was entered, avoid sending an empty message to the server.
            if (messageText == null) return;
            
        	// assemble the message
        	String message =	"<b>" +
        					userName +
        					":</b> " +
        					(String)messageText +
        					"<br><br>";
        					
        	// and send it to the chatChannel
        	Messaging.send("chatChannel", message, new RPCCallback () {
     
				@Override
				public void execute(RPCResponse response, Object rawData,
						RPCRequest request) {
					if (response.getStatus() != RPCResponse.STATUS_SUCCESS) SC.say("Failed send message to server.");
				}
        	});
    }
}
